import ij.*;
import ij.io.*;
import ij.gui.*;
import ij.process.*;
import ij.plugin.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import UroIrmPackage.*;

/** Opens a folder of images DICOM as a stack. */
public class Open_dicom_stack_ implements PlugIn,Header,UroIrmMessage {

	private static boolean grayscale,halfSize;
	private static boolean[] BresultCheckboxImage;
	private int n, start;
	private String text = "",textTitle ="" ;
	private String properties = "";

	private String[] textPreferencies = {"Convert to 8-bits","Open 1/2 Size"};
	private boolean[] BPreferencies ={grayscale,halfSize};

	private DicomHashtable imageTable = new DicomHashtable();
	DicomPref dico = new DicomPref();
	private String key;
	private String class_name = "Open_dicom_stack ";
		
		
	public void run(String arg) {
		PrefSaveResult.load();
		if (!Open(arg))
			return;
		IJ.run("Brightness/Contrast...");
		
	}
	
/** opens all the files in a directory and try to classify them . */
	boolean Open(String arg){
		OpenDialog od = new OpenDialog("Image Sequence...", arg);
		String directory = od.getDirectory();
		String name = od.getFileName();
		if (name==null)
			return false;
		String[] list = new File(directory).list();
		if (list==null)
			return false;
		IJ.register(FolderOpener.class);
		ij.util.StringSorter.sort(list);
		if (IJ.debugMode) IJ.log("FolderOpener: "+directory+" ("+list.length+" files)");
		ImageStack stack = null;
		int width = 0 , height =0 , type=0;
		double min = Double.MAX_VALUE;
		double max = -Double.MAX_VALUE;
		
		try {
			int errNb = 0 ; boolean[] errType = null; String errMessage="";
			for (int i=0; i<list.length; i++) {
				 if ( endsDocumentText(list[i]) ) 
					continue;
				String path = directory + Prefs.separator + list[i];
				ImagePlus imp = (ImagePlus)IJ.runPlugIn("ij.plugin.DICOM", path);
				try {
					if (imp!=null) 					 
						imageTable.addImage(imp);
				} catch( NullKeyFieldException e ) {
						errNb ++;
						if ( errType == null)
							errType = new boolean[e.indice.length];
						boolean newData = false;
						int index = (errMessage.lastIndexOf("\n")== -1)?0:errMessage.lastIndexOf("\n")+1;
						String errLine = errMessage;
						for (int j=0;j<e.indice.length;j++){
							if ( !errType[j]&& e.indice[j]) {
								newData = true;
								if ( errLine.length() > (msg_err_key_field_part1.length()-2) ) {
									errMessage += errLine + "\n" ;errLine ="";
								}  
								errLine += DicomHashtable.KEY_TOKEN[j] + " ; ";
								errType[j]= true;
							}
						}
						if ( newData)
							errMessage += errLine;
				}
			}

			if ( errNb > 0 ){
				if ( !showErrDialog(errNb,errMessage) )
					return false;
			}

			if ( !showDialog(imageTable))
				return false;

			LinkedList listImage = imageTable.getList(key);
			int nList = listImage.size()-1;
			if ( nList <2 ){
				IJ.error(msg_err_no_stack);	 
				return false;
			}
			if (n<1)
				n = nList;
			if (start<1 || start>nList)
				start = 1;
			if (start+n-1>nList)
				n = nList-start+1;

			int count = 0;
			for (int i=start; i<nList+1; i++) {
				text = "";
				ImagePlus imp = (ImagePlus)(listImage.get(i));
				if (stack==null) {

					textTitle = "";
					textTitle+=setTitle(imp);
					properties += "------------------ Information from the PARAMETRE ------------------ \n";
					properties += "Date format : "+ HeaderDicom.DATE_TYPE + "\n";
					properties += setProperties("Stack",imp,0);
					width = imp.getWidth();
					height = imp.getHeight();
					type = imp.getType();
					ColorModel cm = imp.getProcessor().getColorModel();
					if (halfSize)
						stack = new ImageStack(width/2, height/2, cm);
					else
						stack = new ImageStack(width, height, cm);
				}
				text += setHeader(imp);
				count = stack.getSize()+1;
				properties += setProperties("Slice",imp,count);
				IJ.showStatus(count+"/"+n);
				IJ.showProgress((double)count/n);
				ImageProcessor ip = imp.getProcessor();
				if (grayscale)
					ip = ip.convertToByte(true);
				if (halfSize)
					ip = ip.resize(width/2, height/2);
				if (ip.getMin()<min) min = ip.getMin();
				if (ip.getMax()>max) max = ip.getMax();
				stack.addSlice(text, ip);
				if (count>=n)
					break;
			}
	
		} catch(OutOfMemoryError e) {
			IJ.outOfMemory("UroIrmFolderOpener");
			stack.trim();
			return false;
		}
		
		if (stack!=null && stack.getSize()>0) {

			 ImagePlus imp2 = new ImagePlus(textTitle, stack);
			 String[] result = imageTable.getPreferencies(key);
			 String stackTag=""; 
			 for (int i=0;i<result.length;i++){
				stackTag+=   result[i] + "\n";
			 }
			 imp2.setProperty("Info",properties);
			if (imp2.getType()==ImagePlus.GRAY16 || imp2.getType()==ImagePlus.GRAY32)
				imp2.getProcessor().setMinAndMax(min, max);
			imp2.show();

		}
		IJ.showProgress(1.0);
		return true;
	}

/** return true if the String ends with one of the String of "donnee" */
	boolean endsDocumentText(String txt){
		boolean res = false;
		String[] donnee = {".txt",".doc",".xls",".ppt",".pot",".pps"};
		for (int i=0;i<donnee.length;i++)
			res |= txt.endsWith(donnee[i]);
		return res;	
	}


/** open a error dialog windows : "nb" is the number of errors et "message" the message
	to display */
	boolean showErrDialog(int nb, String message){
		Frame current = new Frame();
		ImageJ ij = IJ.getInstance();
		if (ij != null){
			Image img = ij.getIconImage();
			if (img!=null) current.setIconImage(img);
		}
		String errText = THERE_ARE ;
		errText +=  String.valueOf(nb) ; 
		errText += msg_err_key_field_part1;
		errText += center(message,errText.length()) + "\n" ;
		errText += msg_err_key_field_part2;
		int res = JOptionPane.showConfirmDialog(current,
							errText,
							msg_err_key_field_title,			
							JOptionPane.WARNING_MESSAGE,
							JOptionPane.YES_NO_OPTION);
		return ((res==0)?true:false);
	}

	private String center(String text, int size){
		String res = "";
		int length = (size-text.length())/2;
		for (int j=0;j<length;j++){res+=" ";}
		return (res+text+res);
	}

/** display the results of a DicomHashtable */
	boolean showDialog(DicomHashtable table) {
		if ( table == null)
			return error(msg_err_null_pointer);
		int fileCount ;
		String[] stackLabel = initLabel();
		int column = 2;
		int rows = (STACK_TAG.length /column) + 1;
		FolderOpenerDialog gd = new FolderOpenerDialog("Sequence Options",table);
		fileCount = gd.getFileCount();
		if ( !gd.addChoiceTable()) return error(msg_err_prgm + "\n"+msg_err_no_key);
		gd.addMessage(msg_comm_nb_image);
		gd.addNumericField("total :",fileCount , 0);
		gd.addNumericField(msg_comm_start, 1, 0);
		gd.addMessage(msg_comm_preferencies);
		gd.addCheckboxGroup(1,2,textPreferencies, BPreferencies);
		gd.addMessage(msg_comm_option_graph);
		gd.addCheckboxGroup(rows,column,stackLabel,BresultCheckboxImage);
		gd.addMessage("10000 x 10000 x 1000 (100.3MB)");
		gd.showDialog();
		if (gd.wasCanceled())
			return false;
		key = gd.getChoiceTable();
		if (key == null) return error(msg_err_prgm + "\n"+msg_err_no_key);
		n = (int)gd.getNextNumber();
		start = (int)gd.getNextNumber();
		int openstack = 0;
		grayscale =	gd.getNextBoolean();openstack |= (grayscale)?(1<<0):0;
		halfSize =gd.getNextBoolean();openstack |= (grayscale)?(1<<1):0;
		for ( int i = 0; i< BresultCheckboxImage.length; i++){
			BresultCheckboxImage[i] = gd.getNextBoolean();
			openstack |= (BresultCheckboxImage[i] )?(1<<(i+2)):0;
		}
		PrefSaveResult.save("openstack",openstack);
		return true;
	}

// return a table of the label to put in the CheckBoxGroup 	
	private String[] initLabel(){
		int confidential = PrefSaveResult.getInt("confidentiel",1);
		int openstack =PrefSaveResult.getInt("openstack",0);
		if ( confidential == 0 ) {
			BresultCheckboxImage = new boolean[STACK_TAG.length];
			for ( int i =0 ; i<STACK_TAG.length;i++){
				if ( (openstack & (1<<(i+2))) !=0 )
				 BresultCheckboxImage[i] = true;
			}
			return STACK_TAG;
		}
		String[] res = new String[STACK_TAG.length-1];
		BresultCheckboxImage = new boolean[res.length];
		int count =0;
		for (int j=0;j<res.length;j++){
			if ( STACK_TAG[count].equals(PERSONN))
				count++;
			res[j] = STACK_TAG[count];
			if ( (openstack & (1<<(count+2))) !=0)
				BresultCheckboxImage[count] = true;
			count++;
		}
		return res;
	}
	
// returns the header of each image of the stack
	private String setHeader(ImagePlus imp){
		HeaderDicom inf = new HeaderDicom(imp);
		String header ="";
		if(inf.addTag(ACQTIME))
			header += inf.getSlice(); 
		return header;
	}

// returns the title of the new stack
	private String setTitle(ImagePlus imp){
		String title ="";
		HeaderDicom inf = new HeaderDicom(imp);
		inf.addTagStack("");
		boolean type = (BresultCheckboxImage.length == STACK_TAG.length);
		int count = 0;
		for ( int i = 0; i< BresultCheckboxImage.length; i++){
			if ( type && BresultCheckboxImage[i] ) 
				inf.addTagStack(STACK_TAG[i]);
			else if ( !type ){
				if ( STACK_TAG[count].equals(PERSONN))
					count++;
				if ( BresultCheckboxImage[i] ) 
					inf.addTagStack(STACK_TAG[count]);
				count++;
			}	
		}
		title += inf.getStack();
		return title;		
	}

/** set the informations chosen by "OpenParamtere" in the tag "Info" of the stack */
	String setProperties(String type,ImagePlus imp,int image){
		String res ="",title ="",textImage="",info="";
		HeaderDicom inf = new HeaderDicom(imp);
		LinkedList list = dico.getDicomListValue(DicomPref.defaultValue);
		if (type.equals("Stack")){
			title += "------------------ STACK INFORMATION ------------------\n";
		 }else if ( type.equals("Slice")){
			if  ( image == 1 ) 
				title += "------------------ SLICE INFORMATION ------------------\n";
			textImage += " -> IMAGE n " + String.valueOf(image) + " \n" ;
		}
		for (int j =0 ; j< list.size();j++) {
			String[] s = (String[]) list.get(j);
			String current = inf.key2TagImage(s[1],s2i(s[3])) ;
			if (type.equals("Stack") &&  s[2].equals("1"))  {
				info += current;	res += "   " + s[0] + " = " + current  + " \n"; 
			}else if ( type.equals("Slice") && (s[2].equals("2") ||s[2].equals("0")) ){
				info += current;	res += "      " + s[0] + " = " + current + " \n"; 
			}
		}
		if ( info.equals("") ) {
			if ( (type.equals("Slice") && (image ==1)) || type.equals("Stack") )
				return title + " No information \n";	
			else 
				return "";
		}
		return title + textImage +res;
	}

// convert a string in an integer : WARNING it returns -1 if a NumberFormatException is catch
	private int s2i(String s){
		try{ return( (new Integer(s)).intValue());}
		 catch( NumberFormatException e ){return -1;}
	}

/** in order to display general error */	
	boolean error(String txt ){
		IJ.error(class_name + "\n"+ txt);
		return false;
	}

}


/** *******************************************************************************************
		CLASS FOLDER OPENER DIALOG
***********************************************************************************************
*/
class FolderOpenerDialog extends GenericDialog implements UroIrmMessage {
	DicomHashtable table;
	int fileCount;
	int tableCount;
	String[] keys ;
 	boolean eightBits, halfSize;
 	String saveFilter = "";
	private boolean enter; 

	private String[] Champ = DicomHashtable.KEY_TOKEN;
	private boolean[] ChampBool;
	private String sep = "|";

	public FolderOpenerDialog(String title, DicomHashtable table) {
		super(title);
		PrefSaveResult.load();
		this.table = table;
		tableCount= table.size();
		keys = (tableCount==0)?null:new String[tableCount];
		for (int i=0; i< (tableCount);i++){
			keys[i]=table.getKey(i+1);
		}
		if ( keys!=null)
			fileCount = (table.getList(keys[0])).size() -1 ;
		else 
			fileCount = 0;
	}

// convert a string in an integer : WARNING it returns -1 if a NumberFormatException is catch
	private int s2i(String s){
		try{	return( (new Integer(s)).intValue());} 
		catch( NumberFormatException e ){	return -1;}
	}
	
/** convert a StringTokenizer in an array of String */
	String[] convertTokenizer2Array(StringTokenizer chaine){
		if ( chaine.countTokens() <= 0 )
			return null;
		String[] result = new String[chaine.countTokens()];
		for (int i =0; i< result.length;i++){
			result[i] = chaine.nextToken();
		}
		return result;
	}

/** add the current table to the windows : 
	it display a label and a choice( if there isn't one key). */ 
	boolean addChoiceTable(){
		String title ="";
		if (table.hasError()){
			title= msg_err_value_erroneous + table.getErrorKey()+"\n\n";
		}
		if ( table.size() == 1 ) {
			title+= singleElementTable(table.getKey(1));
			addMessage(title);
			return true;
		}
		if ( !analyzerKeys() ) {
			return false;
		}
		title += THERE_ARE ;
		title += (new Integer(tableCount)).toString();
		title += msg_comm_end_stack;
		title += getLabelChoiceCommun() + "\n";
		title += getLabelChoiceDifferent() +  "\n";
		String [] text = new String[tableCount];
		for (int i=0; i< (tableCount);i++){
			text[i] = getValueChoiceTable(keys[i]);
		}
		addMessage(title);
		addChoice(null,text,text[0]);
		((Choice)choice.elementAt(0)).addItemListener(this);
		return true;
	}

/** it initializes the array of booleans : ChampBool thanks to the different
	field of each key ( see unmountKey)*/	
	boolean analyzerKeys(){
		if ( keys == null )
			return false;
		String[] element = table.unmountKey(keys[0]);
		String[] courant = new String[element.length];
		ChampBool = new boolean[Champ.length];
		for (int i=1;i<keys.length;i++){
			courant = table.unmountKey(keys[i]);
			if ( courant.length != element.length ) 
				return false;
			for (int j = 0 ; j< Champ.length;j++){
				if ( ChampBool[j] || !(courant[j]).equals(element[j]) )  
						ChampBool[j] = true;			
			}
		}
		return true;
	}

/** returns the text that correspond to the difference between the keys of the Hashtable*/
	String getLabelChoiceDifferent(){
		if ( keys == null)
			return null;
		boolean confidential = (PrefSaveResult.getInt("confidentiel",1)!=0);
		String text = "";
		text += msg_comm_open_diff + sep  ;
		for (int j = 0 ; j< ChampBool.length;j++){
			if ( ChampBool[j]) {
				String value = keyToString(keys[0],Champ[j]);
				if ( confidential && (Champ[j].equals(DicomHashtable.PERSONN)) )	
					text += DicomHashtable.PATIENTID +sep;
				else
					text += Champ[j] + sep ;
			}
		}
		return text;
	}
	
/** return the text of the commun points between the keys of the Hashtable*/
	String getLabelChoiceCommun(){
		if ( keys == null)
			return null;
		boolean confidential = (PrefSaveResult.getInt("confidentiel",1)!=0);
		String text = "";
		text += msg_comm_open_comm +"\n";
		String current ="";
		int MAX_WIDTH_LABEL = 30;
		for (int j = 0 ; j< Champ.length;j++){
			if ( !ChampBool[j]) {
				String value = keyToString(keys[0],Champ[j]);
				if ( confidential && (Champ[j]).equals(DicomHashtable.PERSONN) )	
					current += "  * " + DicomHashtable.PATIENTID + ":" + table.getPatientID(keys[0]) ;
				else
					current += "  * " + Champ[j] + ":" + value ;
				if ( current.length() > MAX_WIDTH_LABEL ) {
					text+= current + "\n" ;
					current="";
				}
			}
		}
		if ( text.equals(msg_comm_open_comm +"\n") && current.equals(""))
			text += NONE;
		return (text + current);
	}

/** returns the text of the "key" to be added in a ChoiceBox */
	String getValueChoiceTable(String key) {
			int count = (table.getList(key)).size()-1;
			String text = String.valueOf(count) + sep ;
			for ( int j=0;j<ChampBool.length;j++){
				if ( ChampBool[j]) {
					String value = keyToString(key,Champ[j]);
					text += value + sep ;
				}
			}
		return text;
	}
	
/** returns a nice representation of the key of the Hashtable */
	String keyToString(String key, String valeur){
		String[] arrayKey = table.unmountKey(key);
		String[] token = DicomHashtable.KEY_TOKEN;
		if ( !valeur.equals(DicomHashtable.TYPE) ){
			 for (int i=0;i<token.length;i++){
				if ( valeur.equals(token[i])) 
					return (arrayKey[i].equals("null")?UNKNOWN:arrayKey[i]);	
			}
		}
		String text="";
		int i;
		for (i=0;i<token.length;i++){
			if ( (token[i]).equals(DicomHashtable.TYPE))
				break;				
		}
		int type = s2i(arrayKey[i]);
		switch (type)
		 {
			case ImagePlus.COLOR_256: text+=COLOR_256;
						 break;
			case ImagePlus.COLOR_RGB: text += RGB; 
						break;
			case ImagePlus.GRAY8: text+=GRAY8;
						 break;
			case ImagePlus.GRAY16: text += GRAY16; 
					break;
			case ImagePlus.GRAY32: text+=GRAY32;
						 break;
			default : text += NONE ; 
		}
		return text;
	}

// returns the text in the case that the table has only one key
	private String singleElementTable(String key){
			int MAX_WIDTH_LABEL = 30;
			boolean confidential = (PrefSaveResult.getInt("confidentiel",1) !=0);
			int type = s2i(table.getType(key));
			String text = THERE_IS;
			String current ="";
			fileCount = (table.getList(key)).size() -1 ;
			text += ONE_STACK ;
			current += " *  " + NB_IMAGE + (new Integer(fileCount)).toString() ;
			if ( current.length() > MAX_WIDTH_LABEL ) {
				text+= current + "\n" ;
				current="";
			}
			for (int j = 0 ; j< Champ.length;j++){
				String value = keyToString(keys[0],Champ[j]);
				String textValue = Champ[j] ;
				if ( confidential && (Champ[j]).equals(DicomHashtable.PERSONN) )	{
					value = (table.getPatientID(keys[0])==null)?UNKNOWN: table.getPatientID(keys[0]);
					textValue=DicomHashtable.PATIENTID;
				}
				current += "  * " + textValue + ":" + value ;
				if ( current.length() > MAX_WIDTH_LABEL ) {
					text+= current + "\n" ;
					current="";
				}
			}
			text += current ;
			return text;
	}

/** returns the number of files counted */
	public int getFileCount(){
		return fileCount;
	}

/** returns the text of the choice of ChoiceBox .
	if the table contains only one key it returns this key */	
	public String getChoiceTable(){
		if ( choice == null) 
			return  ((keys == null)?null:keys[0]);
		int selected = ((Choice)choice.elementAt(0)).getSelectedIndex();
		if ( selected < 0 ) 
			selected =0 ;
		return keys[selected];
	}

/** returns the number of the "Numericfield" */
	public int getNumber(Object field) {
		TextField tf = (TextField)field;
		String theText = tf.getText();
		double value;
		Double d;
		try {d = new Double(theText);}
		catch (NumberFormatException e){
			d = null;
		}
		if (d!=null)
			return (int)d.doubleValue();
		else
			return 0;;
      }

// Now it is for the interaction of the window

	protected void setup() {
		setStackInfo();
	}
 	
	public void itemStateChanged(ItemEvent e) {
 		setStackInfo();
	}
	
	public void textValueChanged(TextEvent e) {
 		setStackInfo();
	}

	void setStackInfo() {
		String key = getChoiceTable();
		int width = (key==null)?-1:s2i(table.getWidth(key));
		int height = (key==null)?-1:s2i(table.getHeight(key));
		if ( width == -1 ||height == -1 )
				return;
		int type = s2i(table.getType(key));
		int bytesPerPixel = 1;
 		eightBits = ((Checkbox)checkbox.elementAt(0)).getState();
 		halfSize = ((Checkbox)checkbox.elementAt(1)).getState();
 		int n = getNumber(numberField.elementAt(0));
		int start = getNumber(numberField.elementAt(1));
		if (n<1)
			n = fileCount;
		if (start<1 || start>fileCount)
			start = 1;
		if (start+n-1>fileCount) {
			n = fileCount-start+1;
			//TextField tf = (TextField)numberField.elementAt(0);
			//tf.setText(""+nImages);
		}
		// IJ.write(nImages+" "+startingImage);
		switch (type) {
			case ImagePlus.GRAY16:
				bytesPerPixel=2;break;
			case ImagePlus.COLOR_RGB:
			case ImagePlus.GRAY32:
				bytesPerPixel=4; break;
		}
		if (eightBits)
			bytesPerPixel = 1;
		if (halfSize) {
			width /= 2;
			height /= 2;
		}
		double size = (double)(width*height*n*bytesPerPixel)/(1024*1024);
 		((Label)theLabel).setText(width+" x "+height+" x "+n+" ("+IJ.d2s(size,1)+"MB)");
	}


}




