import ij.*;
import ij.gui.*;
import ij.measure.*;
import ij.process.*;
import ij.plugin.frame.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import UroIrmPackage.*;

public class UroIrmGraph_ extends PlugInFrame implements MouseListener,ActionListener, Measurements, GraphInterface  {
// General declaration
	static Frame instance;
	ImagePlus imp;
	PIDGraph pidGraph;
	UroIrmResultDisplay resultTable;
	boolean boolCrossHair = false , boolGoTo = false,boolDelete =false, boolEval =false;
	
	UroIrmPlot plot  ; 
	private Hashtable Btable;
	Panel panelC;
	Label comment1 ,comment2;
	GridBagLayout gridbag;
	GridBagConstraints c;
	private int y = 0,indexPoidsY =0 ;
	
	static int DEFAULT =0 , CROSSHAIR = 1;
	private String titleResult;

	public UroIrmGraph_() {
		super("URO-IRM graph ");
		if ( instance != null ) {
			instance.toFront();
			return;
		}
		instance = this;
		initFrame();
		if ( !init() ) {	
			instance = null;
		}	
	}
	
	 
	public void run ( String arg ){
		if (instance == null){
			setVisible(false);
			WindowManager.removeWindow(this);
			return;
		}
		if ( arg.indexOf("TITLE") >= 0 ) {
			int index = arg.indexOf(":") +1;
			arg = arg.substring(index);
		} 
		setWindowName(arg);
	}

/** create the window with a plot, the buttons of the plots , and the message of the plot*/
	protected boolean init(){
		imp= getImage();	
		if (imp==null)
			return false;

		resultTable = new UroIrmResultDisplay();
		pidGraph = new PIDGraph(imp,resultTable);
		 if ( ! pidGraph.init() )
			return false;
		plot=pidGraph.getPlot();
		//Font monoFont = new Font("Monospaced", Font.PLAIN, 12);
		//Font sanFont = new Font("SansSerif", Font.PLAIN, 12);
		gridbag = new GridBagLayout();
		c = new GridBagConstraints();
		setLayout(gridbag);
	// plot
		addPlot(plot);
	// the buttons	
		initButton();
		pidGraph.setButtonTable(Btable);
		addButtonGroup(BGroup1,BoolGroup1,"plot Option");	// for deleting and readd
		addButtonGroup(BGroup2,BoolGroup2,"zoom Option");	// for the zoom	
		addButtonGroup(BGroup3,BoolGroup3,"Parametre ");	// for the other command
		addButtonGroup(BGroup4,BoolGroup4,null); 			// quit button 
	// le commentaire	
		comment1 = new Label("",Label.CENTER);
		comment2 = new Label("",Label.CENTER);
		addComment(comment1,pidGraph.setComment());
		addComment(comment2,NONE_POINT_SELECTED);
	// link with the class plot
		plot.comment = comment2;
		
		setVisible(true);
		return true;
	}

/** initialize the frame , putting the size, the color */
	protected void initFrame(){
		if (instance == null)
			return;
		Dimension dimScreen = (Toolkit.getDefaultToolkit()).getScreenSize();
		int screenWidth = 500 , screenHeight = 400;
		if ( dimScreen.width < screenWidth )
			screenWidth = dimScreen.width ;
		if ( dimScreen.height < screenHeight)
			screenHeight= dimScreen.height;
		instance.setSize(screenWidth,screenHeight);
		instance.setBackground(Color.lightGray);
	}

	private void setInsets(int[] value){
		c.insets = new Insets(value[0],value[1],value[2],value[3]);
	}

	private void addPlot(Panel plot){
		setInsets(insetsPlot);
		c.gridy = y;c.gridx=0;
		c.weighty=poidsY[indexPoidsY]; c.weightx=100;
		c.gridwidth = 4;c.gridheight =7;
		y+=7;indexPoidsY++;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.CENTER;
		gridbag.setConstraints(plot, c);
		plot.addMouseListener(this);
		add(plot);
		plot.repaint();
	}

/** create a table of buttons present on the frame */
	void initButton(){
		Btable = new Hashtable();
		for (int i =0  ;i < Bname.length;i++)
			Btable.put(Bname[i],new Button(Bname[i]));
	}
	
/** returns the button of the "key" */
	Button getButton(String key){
		if ( !Btable.containsKey(key))
			return null;	
		return ((Button)Btable.get(key));
	}

/** add a single button on the frame */ 
	void addButton (Button b, boolean enable,Panel panel) {
		b.addActionListener(this);
		b.setEnabled(enable);
		panel.add(b);
	}

	private void addButtonGroup(String[] Blabel, boolean[] Bbool,String Gtitle ){
		setInsets(insetsButton);
		int x = 0;
		c.gridy = y;
		c.weighty=poidsY[indexPoidsY];c.gridheight =1;
		c.gridwidth=1;
		y++;indexPoidsY++;

		if ( Blabel.length == 0 )
			return ;
		int poids = 100/Blabel.length;
		if ( Gtitle != null){
			poids = 100/(Blabel.length+1);
			c.weightx=poids;c.gridx=x;x++;
			add(new Label(Gtitle,Label.CENTER),c);
		}
		Button b;
		for (int i = 0 ; i< Blabel.length;i++){
			c.weightx=poids;c.gridx=x;x++;
			if ( Blabel[i].equals("") ){
				add(new Label ("",Label.CENTER),c);
			} else {
				b = getButton(Blabel[i]);
				if ( b == null)
					continue;
				b.addActionListener(this);
				b.setEnabled(Bbool[i]);
				b.resize(ButtonWidth,ButtonHeight);
				add(b,c);
				b.validate();
			}
		}
	}
	
	private void addComment(Label comment,String text){
		setInsets(insetsComment);
		c.gridy = y;c.gridx=0;
		c.weighty=poidsY[indexPoidsY];c.weightx=100;
 		c.gridheight =1;c.gridwidth=4;
		y++;indexPoidsY++;
		comment.setText(text);
		add(comment,c);
	}

/** change name of the windows if the name is available */
	void setWindowName(String name){
		String title="";
		if ( imp == null ) 
			return;
		ImageWindow win = imp.getWindow();
		String WinTitle = "" + win.getTitle();
		int index = WinTitle.indexOf(":");
		if ( instance != null);
			 title += instance.getTitle();
		if ( !name.equals("")){
			title += HeaderDicom.STACK_SEPARATOR + name;
		}
		if ( index > 0 ) 
			title += HeaderDicom.STACK_SEPARATOR + WinTitle.substring(index + 1 );
		else 
			title += HeaderDicom.STACK_SEPARATOR + HeaderDicom.UNKNOWN_PERSON ;
		instance.setTitle(title);
		resultTable.setTitle(name);

	}

/** gets the image ImagePlus */
	ImagePlus getImage() {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) {
			error(msg_err_no_image);
			return null;
		} else
			return imp;
	}

/** fonction due to the action of the buttons */
	public void actionPerformed( ActionEvent e ){
		String label = e.getActionCommand();
		if ( label == null ) 
			return;
		String command = label ;
		if ( command.equals(b_delete) ) 
			deletePoint();
		else if ( command.equals(b_readd) ) 
			readd();
		else if ( command.equals(b_evaluate) ) 
			evaluate();
		else if ( command.equals(b_zoom))
			zoom();
		else if ( command.equals(b_dezoom))
			dezoom();
		else if ( command.equals(b_cancelZoom))
			reInitZoom();
		else if (command.equals(b_result)) 
			displayResultTable(); 
		else if ( command.equals(b_goto))
			goToImage();
		else if ( command.equals(b_redraw))
			redraw();
		else if( command.equals(b_quit))
			Quit();
		else if (command.equals(b_info))
			stackInfo();
		else if (command.equals(b_optionGraph))
			setOptionGraph();
		
	}
	
	
/** switch the cursor type */
	void changeButton(int type,String button) {
		if ( type == DEFAULT ) {
			if (boolCrossHair) {
				super.setCursor(super.DEFAULT_CURSOR);
				boolCrossHair=false;
			}
		}
		if (type == CROSSHAIR ) {
			if (!boolCrossHair){
				super.setCursor(super.CROSSHAIR_CURSOR);
				boolCrossHair=true;
			}
		}
	
		if ( button.equals(b_goto) ){
			boolGoTo = true;  boolDelete = false; boolEval =false;
		} else if ( button.equals(b_delete ) ){
			boolGoTo = false;  boolDelete = true; boolEval=false;
		} else if ( button.equals(b_evaluate)){
			boolGoTo = false;  boolDelete = false; boolEval = true;
		} else {
			if ( button.equals(b_readd))
				resultTable.upDate();
			boolGoTo = false;  boolDelete = false; boolEval=false;
			comment1.setText(pidGraph.setComment());
			comment2.setText(msg_comm_no_point_selected);
		}

	} 

/** redraw the image */
	protected void reDraw () {
		String txt = pidGraph.reDraw();
		if ( txt != null) comment2.setText(txt);
		plot.repaint();
		remove(plot);
		add(plot);
	}


// les differentes fonctions liees aux actions

/** to delete a point of the plot */
	void deletePoint () {
		changeButton(DEFAULT,b_delete);
		pidGraph.deletePoint();
	}	
	
/** readd a point in the plot */
	void readd () {
		changeButton(DEFAULT,b_readd);
		String result = pidGraph.readd();
		comment2.setText(msg_comm_no_point_selected);
		if (result != null)
			comment1.setText(result);
		else
			comment2.setText(msg_comm_no_point);
		reDraw();
	}

/** take 2 points of the graph, and evaluate the aire made of the triangle made by this 
	2 points */
	void evaluate() {
		changeButton(DEFAULT,b_evaluate);
		comment1.setText(msg_comm_first_evaluation);
		comment2.setText(pidGraph.evaluationClear());
	}

/** launch the frame to display the value of the points of the plot */
	void 	displayResultTable () {
		changeButton(DEFAULT,b_result);
	// set the title for the result
		resultTable.upDateAndVisible();	
		pidGraph.displayResultTable();	
	}

/** change the Scale of the plot */
	void setOptionGraph(){
		changeButton(DEFAULT,b_optionGraph);
		String txt = pidGraph.setOptionGraph();
		if ( txt !=null) comment2.setText(txt);
		repaint();
		instance.setVisible(false);
		instance.setVisible(true);
		reDraw();
	}

/** set the current image that was pointed by the mouse*/
	void goToImage(){
		changeButton(DEFAULT,b_goto);
		String txt = pidGraph.goToImage();
		if (txt != null)comment2.setText(txt);
	}

/** cancel all zoom  */
	void reInitZoom() {
		changeButton(CROSSHAIR,b_cancelZoom);
		String txt = pidGraph.reInitZoom();
		if ( txt !=null)comment2.setText(txt);
		reDraw();
	}

/** zoom or not */
	void zoom(){
		changeButton(CROSSHAIR,b_zoom);
		String result = pidGraph.zoom();
		if (result !=null) comment1.setText(result);
		reDraw();
	}	

/** cancel last zoom */ 
	void dezoom () {
		changeButton(CROSSHAIR,b_dezoom);
		String txt = pidGraph.dezoom();
		if (txt !=null) comment2.setText(txt);
		reDraw();
	} 
	
/** redraw the plot */
	void redraw(){
		changeButton(DEFAULT,b_redraw);	
		reDraw();	
	}

/** display the stackInfo */
	void stackInfo(){
		changeButton(DEFAULT,b_info);
		IJ.runPlugIn("ij.plugin.filter.Info","");
	}


/** clear the Worksheet */
	boolean clearWorksheet(){
		if ( !resultTable.getUnsaved())
			return true;
		SaveChangesDialog d = new SaveChangesDialog(IJ.getInstance(), 
							msg_question_save_result);
		if (d.cancelPressed())
			return false ;
		else if (d.savePressed()){
			resultTable.end();
		}
		 return true;
	}
	
/** quit the frame */
	void Quit(){
		changeButton(DEFAULT,b_quit);
		if  ( !clearWorksheet() ) 
			return ;
		setVisible(false);
		WindowManager.removeWindow(this);
		instance = null;
	}

	public void windowClosed(WindowEvent e) {
		clearWorksheet();
		setVisible(false);
		WindowManager.removeWindow(this);
		instance = null;
	}

// to convert float in int 
	private int f2i(float x ) {
		return((new Float(x)).intValue());
	}
		
/** display the specific error of the frame */
	boolean error(String msg) {
		new MessageDialog(this, name_class , msg);
		return false;
	}

// fonction due to the mouseListener
	public void mousePressed(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	if ( boolDelete ) {
			String result = pidGraph.deletePoint();
			comment1.setText(pidGraph.setComment());
			comment2.setText(result);
			reDraw();
		} else if ( boolGoTo ) {
		 	comment2.setText(pidGraph.goToImage());
		} else if ( boolEval ){
			String[] result = pidGraph.evaluate();
			comment1.setText(result[0]);
			comment2.setText(result[1]);
		}else {
			comment2.setText(pidGraph.setLabelComment());
		}


}
	public void mouseEntered(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
}

/** ******************************************************************************************
**********************************************************************************************
				 CLASS PIDGRAPH
**********************************************************************************************
**********************************************************************************************
*/

class PIDGraph implements Measurements, GraphInterface {
	private ImagePlus imp;
	private UroIrmResultDisplay resultTable;
	private HeaderDicom header;
	private String name_class = " Graph "; 
	private Hashtable Btable;

// for the graph
	private UroIrmPlot plot  ; 
	int MIN_ZOOM = 5 ;
	float[][] zoom ; 
	int counterZoom;
	private boolean boolzoom = false;
	static int MAX_ZOOM = 10;
	static int DEFAULT =0 , CROSSHAIR = 1;
	Hashtable tableMinMax;
	float plotData[][];
	float dataDeleted[][];
	private int counterDelete = -1;
	private float[] minMax;
	private boolean bool_zoom = false ;
	private boolean displayResult = false;
	private float[] firstPointOfEvaluation;

// in oder to get the results
	private ResultsTable rt;
	private int measurements;
	private static float[] umeans = new float[MAX_STANDARDS];


	public PIDGraph(ImagePlus source,UroIrmResultDisplay resultTable){
		this.imp=source;
		header = new HeaderDicom(imp);
		this.resultTable = resultTable;
	}

/** initialize and create the plot */ 
	public boolean init(){
		if ( !openStack(imp) ) 
			return false;
		if ( !getData() ) 
			 return false ;
		initPlot();
		return true;
	}
	
/** return the plot */
	public UroIrmPlot getPlot(){
		return plot;
	}

/** return true if there are points deleted */
	public boolean hasPointDeleted(){
		return (counterDelete >-1);
	}
/** returns the header of the stack */
	public HeaderDicom getHeader(){
		return header;
	}

/** set the table of buttons */
	public void setButtonTable(Hashtable b ){
		Btable = b;
	}
	
	private Button getButton(String key){
		if ( !Btable.containsKey(key))
			return null;	
		return ((Button)Btable.get(key));
	}

/** returns true if were are zooming */
	public boolean getBoolZoom(){
		return boolzoom;
	}
	
	private boolean openStack(ImagePlus imp){
		int nSlices = imp.getStackSize();
		if ( nSlices < 1 ) { 
			error(msg_err_no_stack);
			return false;
		}
		int currentSlice = imp.getCurrentSlice();
		String label = "";
		if ( imp.getRoi() != null){
			ImageStatistics stats ;
			int errornb = 0;
			for (int slice=1; slice<=nSlices; slice++) {
				imp.setSlice(slice);
				header.setSlice(slice);
				if ( !header.isTag(HeaderDicom.ETAT) ) {
					header.setEtat(HeaderDicom.ENABLE);
				} else {
					label= header.getTagValue(HeaderDicom.ETAT);
					if ( label.equals(HeaderDicom.DISABLE)){
						header.switchEtat();	
					}
				} 
				if ( header.getTagValue(HeaderDicom.ACQTIME).equals(UroIrmTime.ANORMAL) ) {
					errornb ++; header.setEtat(HeaderDicom.DISABLE);
				}
				stats = imp.getStatistics(measurements);
				saveResults(stats, imp.getRoi());
			}
			imp.setSlice(currentSlice);
			if ( errornb != 0 ) return error(THERE_IS_ARE + String.valueOf(errornb) + IMAGE_WITH+ "\n" + msg_err_anormal_time);

			return true;		 
		} else {
			error(msg_err_no_roi);
			return false;
		}
	}

	private boolean getData() {
		int indexY = rt.getColumnIndex("Mean");
		int indexTps = rt.getColumnIndex(ACQTPSSEC);
		int indexNB = rt.getColumnIndex(NSLICE);
		if (indexY < 0 || indexTps < 0 || indexNB < 0 ) {
			return false; 
		}
	
		if ( rt.getCounter() > 0 ){
			plotData  = new float[3][rt.getColumn(indexTps).length] ;
			dataDeleted  = new float[3][rt.getColumn(indexTps).length] ;	
			plotData[0] = rt.getColumn(indexTps);
			plotData[1] = rt.getColumn(indexY);		
			plotData[2] = rt.getColumn(indexNB);

			return true;
		} else {
			return false;
		}	
	}

	private void initPlot(){
		plot = new UroIrmPlot(imp,CTPSsec,plotData[0],CINT,plotData[1],plotData[2]);
		float offsetX = (minMax[1] - minMax[0])/10;
		float offsetY = (minMax[3] - minMax[2])/10;
		plot.setMinMax(minMax[0]- offsetX,minMax[1]+offsetX,minMax[2]-offsetY,minMax[3]+offsetY);
		zoom = new float[MAX_ZOOM][4];
		zoom[0][0] = minMax[0]-offsetX ; zoom[0][1] = minMax[1]+offsetX ;
		zoom[0][2] = minMax[2]-offsetY ; zoom[0][3] = minMax[3]+offsetY ; 
		counterZoom = 1;
	}	

/** switch the cursor type */
	void changeButton(int type, String name) {
		if ( type == DEFAULT ) {
			if (boolzoom){
				plot.bzoom=false;
				boolzoom=false;
			}
		}

		if (type == CROSSHAIR ) {
			if (!boolzoom){
				plot.bzoom=true;
				boolzoom=true;
			}
		}
		if ( displayResult && (name.equals(b_delete) || name.equals(b_readd)) ) {
			displayResult = false;
		}  
	} 
	
/** change the status of the button named "name" */	
	void changeStatus( String name , boolean enable){
			Button b = getButton(name);
			if ( b != null) {
				b.setEnabled(enable);
			}
	}

/** returns the text for the comment of the plot */ 
	public String setComment(){
		String Text ="";
		Text += THERE_ARE + String.valueOf((plot.getPlotX()).length) + END_NB_POINT;
		if ( counterDelete < 0 )
			Text += NONE_POINT;
 		else
			Text+=(counterDelete +1) + POINT_DELETED;
		return Text;
	}

/** returns the text for the comment of the plot */ 
	public String setLabelComment(){
		float[] pointSelected = plot.getSelectedPoint();
		if ( pointSelected == null)	
			return msg_comm_no_point_selected;
		String[] axes = plot.getLabelAxes();
		String txt = " Point value ( " ;
		txt += axes[0] + " , " + axes[1] +	", n slice ) : " ;
		txt += pointSelected[0] + " , " ;
		txt += pointSelected[1] + " , " ;
		txt += f2i(pointSelected[2]) ;
		return txt;
	}


// les differentes fonctions liees aux actions
///////////////////////////////////////////////////////////
/** delete a point */
	public String deletePoint () {
		changeButton(DEFAULT,b_delete);
		float[] point = null ; 
		int counter = 0 ,indiceDelete = -1;
		point = plot.getSelectedPoint();
		String txt = setLabelComment();
		int length ;
		if ( point != null ) {
			float[] xplot = plot.getPlotX() ;
			float[] yplot = plot.getPlotY();
			float[] indicePoint = plot.getPlotIndicePoint();
			length = ((xplot.length > yplot.length )? yplot.length :xplot.length);
			float[][] auxplot = new float[3][length-1];
			for (int i = 0 ; i < length -1 ; i ++ ) {
				if ( xplot[i]  != point[0] && yplot[i] != point[1]  )  {
					auxplot[0][counter] = xplot[i] ;
					auxplot[1][counter] = yplot[i];
					auxplot[2][counter] = indicePoint[i];
					counter ++ ;
				} else {  
					if ( xplot[i]>=0  ) 
						indiceDelete = i;
				}
			}
			if ( indiceDelete != -1 ) {
				auxplot[0][length-2] = xplot[length-1] ;
				auxplot[1][length-2] = yplot[length-1];
				auxplot[2][length-2] = indicePoint[length-1];
			} else{ 
				if ( xplot[length-1] == point[0] && yplot[length-1] == point[1]  ){
					indiceDelete = length;
			  	}else {
					IJ.error(msg_comm_no_point_found); 	
					return null;
			 	}
			}
			if ( indiceDelete > -1 ) {
				counterDelete ++ ;
				dataDeleted[0][counterDelete] = xplot[indiceDelete];
				dataDeleted[1][counterDelete] = yplot[indiceDelete];
				dataDeleted[2][counterDelete] = indicePoint[indiceDelete] ;
				plot.resetSelectedPoint();
				header.setSlice(f2i(indicePoint[indiceDelete]));
				header.switchEtat();
				plot.setPlot(auxplot);
			}
			plot.repaint();
			changeStatus(b_readd,true);
			resultTable.upDate();
 			if ( length == 1 )
				changeStatus(b_delete,false);
			txt += ERASE;
 		}
		return(txt);
	}	
	
/** readd the last point deleted */
	public String readd () {
		changeButton(DEFAULT,b_readd);
		float[] xplot = plot.getPlotX() ;
		float[] yplot = plot.getPlotY();
		float[] indicePoint = plot.getPlotIndicePoint();
		int length = ((xplot.length > yplot.length )? yplot.length :xplot.length);
		float[][] auxPlot = new float[3][length+1] ;
		int counter =0 ,counterFin = -1;
		boolean find= false;		
		if (counterDelete > -1){
			for (int i = 0 ; i < length+1 ; i ++ ) {
				if (  (xplot[counter] > -1) && ((dataDeleted[2][counterDelete]  >  indicePoint[counter])  || find ) )  {
					auxPlot[0][i] = xplot[counter] ;
					auxPlot[1][i] = yplot[counter] ;
					auxPlot[2][i] = indicePoint[counter] ;
					counter ++ ;
				} else  {
					if ( xplot[counter] < 0 ){
						counterFin = i ; 
						break;
					} 
						auxPlot[0][i] = dataDeleted[0][counterDelete] ;
						auxPlot[1][i] = dataDeleted[1][counterDelete] ;
						auxPlot[2][i] = dataDeleted[2][counterDelete] ;
						find = true;
				
			 	}
			}
			if (  counterFin >= 0  )  {
					auxPlot[0][counterFin] = dataDeleted[0][counterDelete] ;
					auxPlot[1][counterFin] = dataDeleted[1][counterDelete] ;
					auxPlot[2][counterFin] = dataDeleted[2][counterDelete] ;
					for (int i = counterFin +1 ; i < length ; i++){
						auxPlot[0][i] = -1;
					}
			}		find = true;

			if ( find ) { 
				plot.setPlot(auxPlot);
				header.setSlice(f2i(dataDeleted[2][counterDelete]));
				header.switchEtat();
				dataDeleted[0][counterDelete] = -1;
				dataDeleted[1][counterDelete] =-1;
				dataDeleted[2][counterDelete] =-1;
				counterDelete -- ;
			}
			plot.repaint();
			changeStatus(b_delete,true);
			if ( counterDelete < 0 ) 
				changeStatus(b_readd,false);	
			return(setComment());
		} else {
			return(null);
		}
	}

/** clear the evaluation */
	public String evaluationClear(){
		firstPointOfEvaluation = null ;
		plot.resetSelectedPoint();
		return msg_comm_no_point_selected;	
	}

/** function to evaluate the aire of the triangle made of 2 points of the plot */
	public String[] evaluate (){
		changeButton(DEFAULT,b_evaluate);
		float[] point = plot.getSelectedPoint();
		String[] text = new String[2];
		if ( point == null ){
			text[0] = (firstPointOfEvaluation == null)? msg_comm_first_evaluation:msg_comm_second_evaluation;
			text[1] = msg_comm_no_point_selected;
			return text;
		} 
		text[1] = setLabelComment();	
		if ( firstPointOfEvaluation == null) {
			firstPointOfEvaluation = point;
			text[0] = msg_comm_second_evaluation;
		} else {
			float Theight = (firstPointOfEvaluation[1] - point[1] );
			float Twidth  = (firstPointOfEvaluation[0] - point[0] );
			float res = Theight * Twidth /2;
			res = (res > 0 )? res : -res;
			resultTable.addEvaluation(firstPointOfEvaluation,point,res);
			firstPointOfEvaluation = null;
			text[0] = " result = " + String.valueOf(res);
		}
		plot.resetSelectedPoint();
		return text;
	}


/** function displayResultTable */
	public String displayResultTable () {
		changeButton(DEFAULT,b_result);
		displayResult= true;
		return null;
	}

/** change the Scale of the plot	 */
	public String setOptionGraph(){
		changeButton(DEFAULT,b_optionGraph);
		int digits = 0;
		String[] labelAxes = findLabelAxes();
		Hashtable tableMinMax = new Hashtable();
		initAbscisseTable(tableMinMax,labelAxes[0] );
		initOrdonneeTable(tableMinMax,labelAxes[1] );
		OptionGraphDialog gd = new OptionGraphDialog(TITLE_OptionGraph,tableMinMax,digits  );
		gd.addMessage("abscisse : ");
		gd.addChoicePlot(null,ABSCISSE_CHOICE,labelAxes[0]);
		float[] res = plot.getMinMax();
		gd.addNumericField("min " , res[0] ,digits);
		gd.addNumericField(" max", res[1],digits);
		gd.addMessage("ordonnee : ");
		gd.addChoicePlot(null,ORDONNEE_CHOICE,labelAxes[1]);
		gd.addNumericField("min " , res[2] ,digits);
		gd.addNumericField(" max", res[3],digits);
		gd.showDialog();

		if ( gd.wasCanceled() ) {
			return null;
		}
		float[] minMaxRes = new float[4];
		for (int i = 0 ; i< 4 ; i++) 
			minMaxRes[i] = (float)gd.getNextNumber();
		String choiceAbscisse = (gd.getNextChoice());
		String choiceOrdonnee = gd.getNextChoice();
		if ( minMaxRes[0] > minMaxRes[1] || minMaxRes[2] > minMaxRes[3]  ) {
			IJ.error(msg_err_dimScale);
			return null;
		}
		if ( !choiceAbscisse.equals(labelAxes[0]))
			setNewPlotAbscisse(choiceAbscisse);
		if ( !choiceOrdonnee.equals(labelAxes[1]))
			setNewPlotOrdonnee(choiceOrdonnee);
		plot.resetSelectedPoint();
		plot.setMinMax(minMaxRes);
		plot.setLabelAxes(choiceAbscisse,choiceOrdonnee);
		plot.repaint();
		return (msg_comm_no_point_selected);
	}

	private String[] findLabelAxes(){
		String[] label = plot.getLabelAxes();
		int i;
		for ( i = 0 ; i< ABSCISSE_CHOICE.length;i++){
			if ( (label[0]).equals(ABSCISSE_CHOICE[i]) )
			 	break;
		}
		if ( i == ABSCISSE_CHOICE.length )
			label[0] = ABSCISSE_CHOICE[0];
		
		for ( i = 0 ; i< ORDONNEE_CHOICE.length;i++){
			if ( (label[1]).equals(ORDONNEE_CHOICE[i]) )
			 	break;
		}
		if ( i == ORDONNEE_CHOICE.length )
			label[1] = ORDONNEE_CHOICE[0];
		return label;
	}

	private void initAbscisseTable(Hashtable table,String defaultLabel){
		float sec[] , min[], hour[];
		float[] MinMax = getMinMax(false,0);
		if ( 	defaultLabel.equals(ABSCISSE_CHOICE[0]) ){
			sec = MinMax;min = divArray(MinMax,60);hour = divArray(MinMax,3600);
		} else if ( defaultLabel.equals(ABSCISSE_CHOICE[1]) ){
			sec = multArray(MinMax,60) ;min = MinMax;hour = divArray(MinMax,60);
		} else if ( defaultLabel.equals(ABSCISSE_CHOICE[2]) ){
			sec = multArray(MinMax,3600); min = multArray(MinMax,60);hour = MinMax;
		} else {
			float[] indexMinMax = getMinMax(true,2);
			sec = new float[2]; 
			sec[0] = plotData[0][f2i(indexMinMax[0])-1]; 
			sec[1] = plotData[0][f2i(indexMinMax[1])-1];
			min = divArray(sec,60);
			hour = divArray(sec,3600);
		}
		table.put(CTPSsec,sec);
		table.put(CTPSmin,min);
		table.put(CTPShour,hour);
		table.put(CNB,getMinMax(true,2));
	}
	
	private void initOrdonneeTable(Hashtable table,String label){
			table.put(CINT,getMinMax(false,2));
	}

	private float[] getMinMax(boolean point, int n){
		float[] result = new float[2];
		if ( point) { 
 			float[][] data = plot.getMinMaxElement(); // 2+3 elements
			if (data == null ){
				result[0] = 0; result[1] = plotData[3].length;			
			}
			result[0] = data[0][n];
			result[1] = data[1][n];
		} else {
			float[] data = plot.getMinMax(); // 4 elements
			if ( data == null ) {
				result[0] = zoom[0][0]; result[1] = zoom[0][3];
			}
			result[0] = data[n];result[1]= data[n+1];
		}
		return result;
	}

	private void copy(float[][] source,float[][] dest,int s,int d){
		dest[0][d] = source[0][s];
		dest[1][d] = source[1][s];
		dest[2][d] = source[2][s];
	}

	private void setNewPlotAbscisse(String absc){
		float[][] oldPlot = plot.getPlot();
		float[][] data = new float[3][plotData.length];
		float[][] res = new float[3][oldPlot[2].length];
		if ( absc.equals(CTPSsec) ) 
			data[0] = plotData[0];
		else if ( absc.equals(CTPSmin) ) 
			data[0] = divArray(plotData[0],60);
		else if ( absc.equals(CTPShour) ) 
			data[0] = divArray(plotData[0],3600);
		else if ( absc.equals(CNB))
			data[0] = 	plotData[2];
		data[1] = plotData[1]; data[2] = plotData[2];
		for(int i=0;i<oldPlot[2].length;i++){
			int indice = f2i(oldPlot[2][i])-1;
		      copy(data,res,indice,i);
		}
		plot.setPlot(res);
		for ( int i=0;i<(counterDelete-1);i++){
			int indice = f2i(dataDeleted[2][i])-1;
			copy(data,dataDeleted,indice,i);
		}
	}

	private void setNewPlotOrdonnee(String ordo){
		float[][] oldPlot = plot.getPlot();
		float[][] data = new float[3][plotData.length];
		float[][] res = new float[3][oldPlot[2].length];
		if ( ordo.equals(CINT) ) 
			data[1] = plotData[1];
		data[0] = plotData[0]; data[2] = plotData[2];
		for(int i=0;i<oldPlot[2].length;i++){
			int indice = f2i(oldPlot[2][i])-1;
		      copy(data,res,indice,i);
		}
		plot.setPlot(res);
		for ( int i=0;i<(counterDelete-1);i++){
			int indice = f2i(dataDeleted[2][i])-1;
			copy(data,dataDeleted,indice,i);
		}
	}

	private float[] divArray(float[] num,int den){
		float[] result = new float[num.length];
		for (int i=0;i<num.length;i++)
			result[i] = num[i] / den ;
		return result;
	}

	private float[] multArray(float[] num,int den){
		float[] result = new float[num.length];
		for (int i=0;i<num.length;i++)
			result[i] = num[i] * den ;
		return result;
	}


/** set the current image that was pointed by the mouse */
	public String goToImage(){
		changeButton(DEFAULT,b_goto);
		float[] pointSelected;
		pointSelected = plot.getSelectedPoint() ; 
		if (pointSelected == null )
			return (msg_comm_no_point_selected);

		imp.setSlice(f2i(pointSelected[2]));
		return ( setLabelComment());
	}

/** cancel all zoom  */
	public String reInitZoom() {
		changeButton(CROSSHAIR,b_cancelZoom);
		counterZoom = 1 ;
		plot.resetSelectedPoint();
		plot.setZoom(null);
		plot.setMinMax(zoom[0]);
		changeStatus(b_dezoom,false);
		reDraw();
		return (msg_comm_no_point_selected);
	}

/** zoom or not */
	public String zoom(){
		changeButton(CROSSHAIR,b_zoom);
		if ( plot.getZoom() != null ) {
			float minX,maxX,minY,maxY,aux;
			float[] res = plot.getZoom();
			minX = res[0]; maxX = res[1];
			minY = res[2]; maxY = res[3];
	
			if ( minX > maxX ) { 
				aux = minX;
				minX = maxX ;
				maxX = aux;
			} 
			if ( minY > maxY ) {
				aux = minY;
				minY =maxY;
				maxY = aux;
			} 
			float auxwidth = f2i((maxX - minX) * 100 /plot.getLengthScaleX());
			float auxheight = f2i((maxY - minY )*100 /plot.getLengthScaleY());
			if ( auxwidth < MIN_ZOOM ||  auxheight < MIN_ZOOM  ){
				return(msg_err_zoom_small_dim);
			}

			plot.setMinMax(minX ,maxX,minY,maxY);
			zoom[counterZoom][0] = minX ; zoom[counterZoom][1] = maxX ;
			zoom[counterZoom][2] = minY ; zoom[counterZoom][3] = maxY ;
			counterZoom++;
			plot.setZoom(null);
			plot.resetSelectedPoint();
			changeStatus(b_dezoom,true);
			if ( counterZoom == MAX_ZOOM)
				changeStatus(b_zoom,false);
			reDraw();
			return (msg_comm_no_point_selected);
		} else {
			if ( !boolzoom) 
				changeButton(CROSSHAIR,b_zoom);
			return(COMMENT_ZOOM);
		}
	}	


/** dezoom */
	public String dezoom () {
		changeButton(CROSSHAIR,b_dezoom);
		if (counterZoom > 2 ) {
			counterZoom -- ;
			plot.setMinMax(zoom[counterZoom-1]);
			changeStatus(b_zoom,true);
			reDraw();
		} else {
			if ( counterZoom > 1 ) 
				counterZoom -- ; 
			changeStatus(b_dezoom,false);
			plot.setMinMax(zoom[0]);
			reDraw();
		}
		plot.resetSelectedPoint();
		return msg_comm_no_point_selected;
	} 
	
/** redraw the image */
	public String reDraw () {
		plot.repaint();
		plot.resetSelectedPoint();
		plot.setZoom(null);
		plot.upDate();
		return msg_comm_no_point_selected;
	}



// SAVE RESULTS OF THE STATISTICS
	private  void saveResults(ImageStatistics stats, Roi roi) {
		if ( rt == null ) {
			rt = new ResultsTable();
		}
		rt.incrementCounter();
		int counter = rt.getCounter();
		if (counter<=MAX_STANDARDS) {
			if (umeans==null) umeans = new float[MAX_STANDARDS];
			umeans[counter-1] = (float)stats.umean;
		}
 		
 		int currentSlice = imp.getCurrentSlice();
		header.setSlice(currentSlice);
		String S_acqTime = header.getTagValue(HeaderDicom.ACQTIME);
		UroIrmTime data = new UroIrmTime(S_acqTime);
		
		float donnee = (new Double(stats.mean)).floatValue();
		if ( minMax == null ) {
				minMax = new float[4];
				minMax[0] = data.getValue() ;
				minMax[1] = data.getValue();	
				minMax[2] = donnee;
				minMax[3] = donnee;
		} else {
			if ( data.getValue() < minMax[0] ) 
				minMax[0] = data.getValue() ;
			if ( data.getValue() > minMax[1] )
				minMax[1] = data.getValue();
			if ( donnee < minMax[2] )
				minMax[2] = donnee;
			if ( donnee > minMax[3])
				minMax[3] = donnee;
		}

		rt.addLabel(HeaderDicom.ACQTIME, data.toString());
		rt.addValue(ACQTPSSEC,data.getValue());
		rt.addValue(NSLICE,currentSlice);      
		rt.addValue(ResultsTable.MEAN,stats.mean);

	}

// to convert float in int 
	private int f2i(float x ) {
		return((new Float(x)).intValue());
	}

/** helps the garbage collector */
	void flush(){
		resultTable.flush();
		resultTable =null;
		imp=null;header=null;Btable=null;plot=null;zoom=null ; 
		tableMinMax=null;plotData=null;dataDeleted=null;minMax=null;
		rt=null;umeans=null ;
	}

/** display the specific error of the frame */
	boolean error(String msg) {
		IJ.error(name_class + "\n "+ msg);
		return false;
	}

}

