package UroIrmPackage;

import java.io.*;
import java.util.*;
import ij.*;

/** interact with the file named IJ_PREFDICOM.txt 
	it saves and gets the preferencies find in the differents programmes*/
public class PrefSaveResult implements UroIrmFileDescriptor,UroIrmMessage{
	private Properties dico;
	static Properties pref;	
	private  static String prefsDir;

/** load the preferencies, if it isn't the first time that fonction is called 
	nothing will be done */
	public static boolean load(){
		if ( pref == null) {
			pref = new Properties();
		}
		String homeDir = System.getProperty("user.dir") ;
		String userHome = System.getProperty("user.home");
		String osName = System.getProperty("os.name");
		if (osName.indexOf("Windows",0)>-1)
			prefsDir = homeDir; //ImageJ folder on Windows
		else
			prefsDir = userHome; // Mac Preferences folder or Unix home dir 

		prefsDir +=  separator + PATH_IJ;
		File f1 = new File(prefsDir+separator+NAME);
		
		if ( !f1.exists() || f1.length()==0){
			try{
			    PrintWriter pw = new PrintWriter(new FileWriter(prefsDir+separator+NAME));
			   	for (int i=0;i<UroIrmKey.length;i++){
					pw.println(UroIrmKey[i]);
			    }
			    pw.close();
	 		} 
			catch (IOException e) {  return false; }
	
 		}

		if ( f1.exists() ) {
			FileInputStream f =null;
			try {f = new FileInputStream(prefsDir+separator+NAME);}
			catch (FileNotFoundException e) {f=null;}
			if ( f == null)
				return false;
			try { pref.load(f);f.close();}
			catch(IOException e ) { pref = new Properties(); return false;} 
		}
		return true;
	}
	
/** return a copy of the preferencies */
	public static Properties getPref(){
		return pref;
	}

/** if the "key" exist, it returns the value 
	if not it create it with the "defaultValue" */
	public static int getSetInt(String key,int defaultValue){
		if (pref==null) 
			return defaultValue;			
		String s = pref.getProperty(key);
		if (s!=null) {
			try {	return Integer.decode(s).intValue();}
			catch (NumberFormatException e) {}
		} else {
			setInt(key,defaultValue);
		}
		return defaultValue;
	} 

/** it gets the value of the "key", if the key is not found it returns the
	"defaultValue" */
	public static int getInt(String key, int defaultValue) {
		if (pref==null) 
			return defaultValue;			
		String s = pref.getProperty(key);
		if (s!=null) {
			try {	return Integer.decode(s).intValue();}
			catch (NumberFormatException e) {}
		}	
		return defaultValue;
	}

/** it set the "value" to the specific "key" */
	public static void setInt(String key,int value){
		setString(key,String.valueOf(value));
			
	}

/** it set the "value" to the specific "key" (whether the key exists or not) */
	public static void setString(String key ,String value){
		if ( pref == null)
			load();
		if ( pref.containsKey(key) )
			pref.remove(key);
		pref.put(key,value);
	}

/** set the "value" to the specific key and save all the keys */
	public static boolean save(String key,String value){
		setString(key,value);
		return save();
	}

/** set the "value" to the specific key and save all the keys */
	public static boolean save(String key,int value){
		setInt(key,value);
		return save();
	}

/** save all the keys, writing them in the file NAME */
	public static boolean save(){
		try{
			FileOutputStream fos = new FileOutputStream(prefsDir+separator+NAME);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			PrintWriter pw = new PrintWriter(bos);
			for (Enumeration e = pref.propertyNames();e.hasMoreElements();){
				String current = (String)e.nextElement();
				pw.println(current + "=" + (String)pref.get(current));
			}
			pw.close();
			fos.close();
	 	} 
		catch (IOException e) { return false;	}
		return true;
	}

/** remove all */
	public static void flush(){
		pref.clear();
		pref = null;
	}

}





